****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpetables.sas  
*
* Created (mm/dd/yyyy): 03/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output table for encounter and covariate characteristics.                                        
*   
*  Program inputs:    
*	-DPLocal.&RUNID._all_encounter
*	-DPLOCAL.&RUNID._all_covariates 
*	-DPLOCAL.&RUNID._DenomCounts 
*                                     
*  Program outputs:  
*	-MSOC.&RUNID._descstat
*	-MSOC.&RUNID._descstat_covariates 
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpetables();

%put =====> MACRO CALLED: ms_qrpetables v1.5;

	data _null_;
		format startfollowup enddate date9.;
		startfollowup=intnx('month',&startfollowup.,0,'begin');  *in case not at the start of month;
	    enddate=intnx('month',&enddate.,1,'begin')-1; *in case not at the end of month;
	    numyears=intck('year',startfollowup,enddate)+1;	
		call symputx("numyears",numyears);
	run;

	*Squared datasets;
		*Other strata - race, hispanic, enctype, discharge_disposition, bloodcomp.
		 Continuous var trans_unit not squared;

			*race;
			data _race;
				format race $1.;
			   	do i=0 to 5;
			    	race=i;
			        output;
			    end;
			run;

			*hispanic;
			data _hispanic;
				format hispanic $1.;
			    hispanic="N";
			    	output;
				hispanic="Y";
					output;
				hispanic="U";
					output;
			run;

			*Enctype;
			data _enctype;
				format enctype $2.;
			    enctype="IP";
			        output;
				enctype="ED";
					output;
				enctype="AV";
					output;
			run;

			*Discharge disposition;
			data _discharge;
				format discharge_disposition $1.;
			    discharge_disposition="A";
			        output;
				discharge_disposition="E";
					output;
				discharge_disposition="U";
					output;
			run;

			*Bloodcomp;
			data _bloodcomp;
				format bloodcomp $65.;
			    bloodcomp="RBCs only";
			        output;
			    bloodcomp="Plasma only";
			        output;
			    bloodcomp="Platlets only";
			        output;
			    bloodcomp="Platelets and Plasma";
			        output;
			    bloodcomp="RBCs and Plasma";
			        output;
			    bloodcomp="RBCs and Platelets";
			        output;
			    bloodcomp="RBCs and Plasma and Platelets";
			        output;
			    bloodcomp="Other";
			        output;
			run;

	*loop through each group;	
	%do Group=1 %to &Ngroup.;

		data _null_;
			set infolder.&cohortfile.;
			if _n_ = &Group. then do;
				call symputx('ITGROUP',strip(Group));            

				*Age stratification;
				if missing(agestrat) then do;
					call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
				end;
				else do;
					call symputx("agestrat",agestrat);
				end;
			end;
		run;

	    *creating age group dataset;
	    data _age;
		    numgrp = countw("&AGESTRAT."," ") ;
		    do i=1 to numgrp;
		        AgeGroup=scan("&AGESTRAT",i," ");
		        AgeGroupSort=i;
		        output;
		    end;
		    keep AgeGroup AgeGroupSort;
	    run;

	    /****************************************************************/
	    /* Squaring (or making sure all possible values are represented */
	    /****************************************************************/
	    
	    data _SquaredYearAgeSex;
		    set infolder.&cohortfile.(drop=sex);
			format sex $1.;	
		    format AgeGroup $9.;
		    numgrp = countw("&AGESTRAT."," ") ;

			LOS=.M;
			trans_unit=0;
			
		    do i=1 to numgrp;
		        AgeGroup=scan("&AGESTRAT",i," ");
		        AgeGroupSort=i;
		        do per=1 to &numyears.;
		            *create year variable;
		            date=intnx('year',&startfollowup.,per-1,'begin');
		            year=year(date);
			        	sex="F"; output;
			            sex="M"; output;
			            sex="O"; output;
		        end;
		    end;
		    keep group AgeGroup AgeGroupSort sex year los trans_unit;
	    run;

		proc sql noprint;
			create table _Squaredtable as
		    select base.*,
				r.race,
		        h.hispanic,
				e.enctype,
				d.discharge_disposition,
				b.bloodcomp
			from  _SquaredYearAgeSex as base,
				  _race as r,
		          _hispanic as h,
				  _enctype as e,
				  _discharge as d,
				  _bloodcomp as b;
		quit;

		*Merge squared table with list of encounters;
		proc sql noprint;
			    create table _mstr as 
			    select mstr.*,
			           age.AgeGroupSort
			    from  dplocal.&runid._all_encounter(where=(EXCL=0 and group = "&itgroup.")) as mstr left join
			          _age as age
			    on age.AgeGroup=mstr.AgeGroup;
			quit;

			data _mstr;
			    set _mstr(in=a)
			        _Squaredtable(in=b);
			    if a then do;
			        encounter=1;
			    end;
			    else do;
			        encounter=0;    
			        %do ev = 1 %to %eval(&num_hoi.);
					event&ev.=0;
					%end;
			    end;
			run;

		%macro createNumTableEntries(class=, outfile=,level=);
	        *One record per encounter for the class stratification;
	        proc means data=_mstr nway missing noprint;
	        var encounter %do ev = 1 %to %eval(&num_hoi.); event&ev. %end;;
	        class encounterID &class.;
	        id encounter;
	        output out=_num0(drop=_:) max(encounter)=count_enc
									  %do ev = 1 %to %eval(&num_hoi.);
									  max(event&ev.)=count_event&ev.
									  %end;;
	        run;

	        *One record per patient for the class stratification;
	        proc means data=_num0 nway missing noprint;
	        var count_enc %do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end;;
	        class &class.;
	        output out=&outfile.(drop=_:)  sum(count_enc)=count_enc
										   %do ev = 1 %to %eval(&num_hoi.);
										   sum(count_event&ev.)=count_event&ev.
										   %end;;
	        run;

			%if %length(&class.)=0 %then %do;
		    	data _NumCounts;
		            set &outfile.;
		            format level $3.;
		            Level="&level.";
		        run;
			%end;
		    %else %do;
		    	data _NumCounts;
		            set _NumCounts
		                &outfile.(in=a);
		                if a then do;	
							Level="&level.";
						end;
		        run;
			%end;
	    %mend;

	    *Overall;
	    %createNumTableEntries(class=,outfile=_ALL,level=000);
		
		*strata;
	    %createNumTableEntries(class=year,outfile=_Y,Level=001);
	    %createNumTableEntries(class=Sex,outfile=_S,Level=002);
	    %createNumTableEntries(class=AgeGroup agegroupsort,outfile=_AG,Level=003);

	    %createNumTableEntries(class=race,outfile=_R,Level=110);
	    %createNumTableEntries(class=hispanic,outfile=_H,Level=115);
	    %createNumTableEntries(class=enctype,outfile=_EN,Level=200);
	    %createNumTableEntries(class=discharge_disposition,outfile=_DD,Level=205);

	    %createNumTableEntries(class=trans_unit,outfile=_TU,Level=210);
	    %createNumTableEntries(class=bloodcomp,outfile=_BC,Level=215);


		*Compute Age and LOS metrics;
		%macro createnummetrics(where=, outfile=, col=);

			*One record per encounter for the class stratification;
		    proc means data=_mstr nway missing noprint;
				var age los;
			    class encounterID;
			    id encounter;
			    output out=_num0(drop=_:) max(age)=age
										  max(los)=los;
				where &where.;
			run;

			proc means data=_num0 nway noprint missing;
				var age los;
				output out=_num1(drop=_:) mean(age) = age_mean
										  std(age) = age_std
										  min(los) = los_min
										  p1(los) = los_p1
										  p5(los) = los_p5
										  p25(los) = los_25
										  p75(los) = los_75
										  p95(los) = los_p95
										  p99(los) = los_p99
										  mean(los) = los_mean
										  median(los) = los_md
										  max(los) = los_max ;
			run;

			%isdata(dataset=_num1);
			%if %eval(&nobs.>0) %then %do;
			*Transpose and create analytic dataset;
				proc transpose data=_num1 out=_agemetrics(rename=_name_=age rename=col1=&col.);
					var age:;
				run;
				proc transpose data=_num1 out=_losmetrics(rename=_name_=los rename=col1=&col.);
					var los:;
				run;
			%end;
			%else %do;
				data _agemetrics;
					format age $8.;
					age='age_mean';
					&col.=.;
					output;
					age='age_std';
					&col.=.;
				run;
					
				data _losmetrics;
					format los $8.;
					los='los_min';
					&col.=.;
					output;
					los='los_p1';
					&col.=.;
					los='los_p1';
					&col.=.;
					los='los_p5';
					&col.=.;
					los='los_p25';
					&col.=.;
					los='los_p75';
					&col.=.;
					los='los_p95';
					&col.=.;
					los='los_p99';
					&col.=.;
					los='los_mean';
					&col.=.;
					los='los_md';
					&col.=.;
					los='los_max';
					&col.=.;
				run;
			%end;

			data &outfile.;
		    	set _agemetrics(in=a)
					_losmetrics(in=b);
		        if a then do;	
					Level="220";
				end;
				if b then do;
					Level="225";
				end;
		    run;

			proc sort data=&outfile.; 
				by level age los;
			run;

			%if "&where." = "1" %then %do;
				data _metrics;
					set &outfile.;
				run;
			%end;
			%else %do;
				data _metrics;
					merge &outfile. _metrics;
					by level age los;
				run;
			%end;

			proc datasets noprint nowarn lib=work;
				delete _agemetrics _losmetrics _num0 _num1;
			quit;
		%mend;

		%createnummetrics(where=1, outfile=_allcont, col=count_enc);
		%do ev=1 %to %eval(&num_hoi.);
			%createnummetrics(where=event&ev=1, outfile=_event&ev.cont, col=count_event&ev.);
		%end;

		*Merge with _numcounts;
		data _NumCounts;
			set _NumCounts
				_metrics;
		run;

		*Merge with _denomcounts;
		proc sort data=_NumCounts;
			by Level sex agegroupsort year race hispanic enctype discharge_disposition;
		run; 

		*merge denoms with _age;
		proc sql noprint;
			create table _denomcounts as
			select x.*,
				y.agegroupsort
			from dplocal.&runid._DenomCounts as x
				left join _age as y
			on x.agegroup=y.agegroup
			where group = "&itgroup."
			order by level, sex, agegroupsort, year, race, hispanic, enctype, discharge_disposition;
		quit;

		data _descstat;
			merge _NumCounts (in=a)
				  _DenomCounts (in=b);
			by Level sex agegroupsort year race hispanic enctype discharge_disposition ;
			format group $30.;

			group = "&itgroup.";

			if not b then do;
			DenNumEnc=.;
			DenNumPt=.;
			DenNumFac=.;
			end;

			if b and missing(DenNumEnc) then do;
			DenNumEnc=0;
			DenNumPt=0;
			DenNumFac=0;
			end;

			rename agegroupsort=agegroupnum;
		run;

		*Remove all labels;
		proc datasets lib=work memtype=data nolist nowarn;
		    modify _numcounts;
			modify _descstat; 
		    attrib _all_ label=' '; 
		run;

		* Output to DPLOCAL and MSOC;
		%if %eval(&group.=1) %then %do;
			data DPLOCAL.&RUNID._numcounts;
		        set _NumCounts;
		    run;

			data MSOC.&RUNID._descstat;
				retain group Level sex agegroup agegroupnum year race hispanic enctype discharge_disposition trans_unit bloodcomp age los count_enc
					%do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end; DenNumEnc DenNumPt DenNumFac;
				set _descstat;
				keep group Level sex agegroup agegroupnum year race hispanic enctype discharge_disposition trans_unit bloodcomp age los count_enc
					%do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end; DenNumEnc DenNumPt DenNumFac;
			run;
		%end;
		%else %do;
			proc append base=DPLOCAL.&RUNID._numCounts
				data=_NumCounts force;
		    run;

			proc append base=MSOC.&RUNID._descstat
		    	data=_descstat(keep=group Level sex agegroup agegroupnum year race hispanic enctype discharge_disposition trans_unit bloodcomp age los 
						count_enc %do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end; DenNumEnc DenNumPt DenNumFac) force;
		    run;
		%end;

	%end; *loop through groups;

	/*************************/
	/** Descstat_covariates **/
	/*************************/

	*Must have at least 1 covariate;
	%ISDATA(dataset=infolder.&COVARIATECODES.);	    
	%IF %EVAL(&NOBS.>0) %THEN %DO;

		*square;
		proc sql noprint;
			create table _squaredcovars as
			select cov.*,
				   grp.group
			from studynames as cov,
			infolder.&cohortfile.(keep=group) as grp;
		quit;

		*merge covariates with all_encounters to get event and EXCL status;
		proc sql noprint;
			create table _allcovars as
			select cov.*
				   %do ev = 1 %to %eval(&num_hoi.); 
				  	, enc.event&ev.
				   %end;
			from dplocal.&runid._all_covariates as cov
			inner join dplocal.&runid._all_encounter(where=(EXCL=0)) as enc
			on cov.encounterID = enc.encounterID and cov.group = enc.group;
		quit;

		data _mstrcovars;
			set _allcovars(in=a)
				_squaredcovars;
				if a then do;
					encounter=1;
				end;
				else do;
					encounter=0;
					 %do ev = 1 %to %eval(&num_hoi.);
						event&ev.=0;
					%end;
				end;
		run;

		%macro createcovnums(class=, where=, outfile=,level=);
			*One record per encounter for the class stratification;
		    proc means data=_mstrcovars nway missing noprint;
		        var encounter %do ev = 1 %to %eval(&num_hoi.); event&ev. %end;;
		        class encounterID &class.;
		        id encounter;
		        output out=_num0(drop=_:) max(encounter)=count_enc
										  %do ev = 1 %to %eval(&num_hoi.);
										  max(event&ev.)=count_event&ev.
										  %end;;
				where &where.;
		    run;

		    *One record per patient for the class stratification;
		    proc means data=_num0 nway missing noprint;
		        var count_enc %do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end;;
		        class &class.;
		        output out=&outfile.(drop=_:)  sum(count_enc)=count_enc
											   %do ev = 1 %to %eval(&num_hoi.);
											   sum(count_event&ev.)=count_event&ev.
											   %end;;
		    run;

			%if "&where." ="1" %then %do;
			  	data _NumCovarCounts;
				    set &outfile.;
			        format level $3.;
			        Level="&level.";
			    run;
			%end;
		    %else %do;
		    	data _NumCovarCounts;
		            set _NumCovarCounts
		                &outfile.(in=a);
		                if a then do;	
							Level="&level.";
						end;
		        run;
			%end;
		%mend;

		*Overall;
		%createcovnums(class=group studyname covarnum covartype, where =1, outfile=_allcovs,level=300);
			
		*metrics for MEDADM covars;
		%createcovnums(class=group studyname covarnum covartype days_rx_allow_gap, where =upcase(covartype)="MEDADM" and days_rx_allow_gap is not missing, outfile=_rxallowgap,level=305);
		%createcovnums(class=group studyname covarnum covartype days_rx_nogap, where =upcase(covartype)="MEDADM" and days_rx_nogap is not missing, outfile=_rxnogap,level=310);
		%createcovnums(class=group studyname covarnum covartype codefreq, where =upcase(covartype)="MEDADM" and codefreq is not missing, outfile=_codefreq,level=315);

		proc sort data=_NumCovarCounts;
			by group level covarnum days_rx_allow_gap days_rx_nogap codefreq;
		run; 

		* Output to DPLOCAL and MSOC;
		data DPLOCAL.&RUNID._NumCovarCounts;
			set _NumCovarCounts;
		run;

		data MSOC.&RUNID._descstat_covariates;
			retain group level covarnum studyname covartype days_rx_allow_gap days_rx_nogap codefreq count_enc %do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end; ;
			set _NumCovarCounts;
				keep group level covarnum studyname covartype days_rx_allow_gap days_rx_nogap codefreq count_enc %do ev = 1 %to %eval(&num_hoi.); count_event&ev. %end; ;
		run;

	%end;

%put NOTE: ******** END OF MACRO: ms_qrpetables v1.0 ********;

%mend ms_qrpetables;
